/*
 *    PM6025.h - A library for the DisplayTech PM6025 LCD 
 *    Copyright (c) 2016 Istvan Cserny
 * 
 *    Permission is hereby granted, free of charge, to any person
 *    obtaining a copy of this software and associated documentation
 *    files (the "Software"), to deal in the Software without
 *    restriction, including without limitation the rights to use,
 *    copy, modify, merge, publish, distribute, sublicense, and/or sell
 *    copies of the Software, and to permit persons to whom the
 *    Software is furnished to do so, subject to the following
 *    conditions:
 * 
 *    This permission notice shall be included in all copies or 
 *    substantial portions of the Software.
 * 
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *    OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef PM6025_h
#define PM6025_h

#if (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif


const static uint8_t row1[] = {2, 7, 12, 17, 22, 27, 32, 37, 41, 45, 49, 54, 58};	//Nibble addresses in row 1
const static uint8_t row2[] = {0, 5, 10, 15, 20, 25, 30, 35, 39, 43, 47, 52, 56};	//Nibble addresses in row 2
const static uint8_t row3[] = {60, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 108, 112};	//Nibble addresses in row 3
const static uint8_t pvpv[] = {4, 9, 14, 19, 24, 29, 0, 34, 117, 51,  0,  0,  0};	//Nibble addresses of decimal points/commas


/*
 * Segments to be switched on for digits on the
 * 7-segment display (Bit order: abcx fged)
 * Segment labeling (from left to right): 
 *                   ---   (a)
 *                  |   |  (fb) 
 *                   ---   (g)
 *                  |   |  (ec)
 *                   ---   (d)
 */

const static uint8_t numTable[] = { 0xEB, 0x60, 0xC7, 0xE5, 0x6C, 0xAD, 0xAF, 0xE0, 0xEF, 0xED, 0xEE, 0x2F, 0x8B, 0x67, 0x8F, 0x8E, 0x00, 0x04 };
/*
      abcx fged             abcd efgx 
	0b1110 1011,		//0 1111 1100		
	0b0110 0000,		//1 0110 0000		
	0b1100 0111,		//2 1101 1010
	0b1110 0101,		//3	1111 0010
	0b0110 1100,		//4	0110 0110
	0b1010 1101,		//5	1011 0110
	0b1010 1111,		//6	1011 1110
	0b1110 0000,		//7	1110 0000
	0b1110 1111,		//8	1111 1110
	0b1110 1101			//9	1111 0110
	0b1110 1110			//A 1110 1110
	0b0010 1111			//b 0011 1110
	0b1000 1011			//C 1001 1100
	0b0110 0111			//d 0111 1010
	0b1000 1111			//E 1001 1110
	0b1000 1110			//F 1000 1110	
    0b0000 0000         //SPACE	
	0b0000 0100         //MINUS SIGN
*/

/*
 * Segments to be switched on for characters and digits on
 * 16-segment display (Bit order: cdhj aeko bflp gimn)
 * Segment labeling (from left to right): 
 *                   --- ---   (ab)
 *                  | \ | / |  (cdefg) 
 *                   --- ---   (hi)
 *                  | / | \ |  (jklmn)
 *                   --- ---   (op)
 */
const static uint16_t charTable[48] = {
	0x9bd9, //0: abcfgjknop
	0x0049, //1: fgn
	0x399c, //2: abghijop
	0x099d, //3: abginop
	0xa00d, //4: cghin
	0xa995, //5: abchinop
	0xb915, //6: achijnop
	0x0889, //7: abgn
	0xb99d, //8: abcghijnop
	0xa89d, //9: abcghinp
	0,
	0,
	0,
	0,
	0,
	0,
	0x99dc, //@: abcfgijop
	0xb88d, //A: abcghijn
	0x0dbd, //B: abegilnop
	0x9990, //C: abcjop
	0x0db9, //D: abeglnop
	0xb990, //E: abchjop
	0xb880, //F: abchj
	0x9995, //G: abcijnop
	0xb00d, //H: cghijn
	0x0db0, //I: abelop
	0x1119, //J: gjnop
	0xb042, //K: cfhjm
	0x9110, //L: cjop
	0xd049, //M: cdfgjn
	0xd00b, //N: cdgjmn
	0x9999, //O: abcgjnop
	0xb88c, //P: abcghij
	0x999b, //Q: abcgjmnop
	0xb88e, //R: abcghijm
	0xa995, //S: abchinop
	0x0ca0, //T: abel
	0x9119, //U: cgjnop
	0x9240, //V: cfjk
	0x920b, //W: cgjkmn
	0x4242, //X: dfkm
	0xa02c, //Y: cghil
	0x0bd0, //Z: abfkop
	0,
	0,
	0,
	0,
	0
};

//-- Predefined bit values for controlling sidebar symbols
#define sNull		0x8000U		//Symbol 1
#define sNet		0x4000U		//Symbol 2
#define sBalance	0x2000U		//Symbol 3
#define sBattery	0x1000U		//Symbol 4
#define sV1			0x0080U		//Symbol V1
#define sV2			0x0040U		//Symbol V2
#define sV3			0x0020U		//Symbol V3
#define sV4			0x0010U		//Symbol V4
#define sV5			0x0001U		//Symbol V5

class PM6025 {
 private :
    byte status[60];			//data array to be sent to the LCD
	void setNibble(uint8_t addr, uint8_t data); 
	uint8_t getNibble(uint8_t addr);   
    void flush();										//Update the PM6025 display
	
 public:
	PM6025(void);	
    void init(uint8_t data = 0x00);						//Initialize the PM6025 display (testmode then clear screen)
	void setDigit(uint8_t row, uint8_t pos, uint8_t c); //Display a 7-seg digit   (only in row 1 or 2)
    void setChar(uint8_t pos, char c);					//Display a 16-seg letter (only in row 3)
	void setSymbols(uint16_t flags);					//Display/clear sidebar symbols (S1..S4, V1..V5)
	void setDP(uint8_t row, uint8_t pos);				//Display a decimal comma (pos 1..6, 8..10; row 1 or 2)
	void clearDP(uint8_t row, uint8_t pos);				//Remove a decimal comma (pos 1..6, 8..10; row 1 or 2)
	void write(uint8_t row, uint8_t pos, long data);	//Display signed integers	
};

#endif	//PM6025.h



